#include "stdafx.h"
#include "ImageProxy.h"
#include "Application.h"

/************************************************************************/
/* ImageProxy */
/************************************************************************/
void ImageProxy::LazyInitialize()
{
	//lazy initialize
	if (m_p == NULL)
	{
		m_p = Application::get_instance()->LoadImage(m_ID);
	}
}
void ImageProxy::Free()
{
	//Application::get_instance()->FreeImage(m_ID);
	m_p = NULL;
}
/************************************************************************/
/* CBitmapImage */
/************************************************************************/
CBitmapImage::CBitmapImage()
{
	m_hBuffer = NULL;
}
CBitmapImage::~CBitmapImage()
{
	Unload();
	if (m_hBuffer)
	{
		::GlobalUnlock(m_hBuffer);
		::GlobalFree(m_hBuffer);
		m_hBuffer = NULL;
	}
}
BOOL CBitmapImage::Load(HINSTANCE hResourceModule, UINT uID)
{
	BOOL bSuccess = FALSE;
	Unload();
	m_img.LoadFromResource(hResourceModule, uID);
	bSuccess = !m_img.IsNull();
	return bSuccess;
}
BOOL CBitmapImage::Load(HINSTANCE hResourceModule, UINT uID, LPCTSTR lpType)
{
	BOOL bSuccess = FALSE;
	Unload();
	m_img.LoadFromResource(hResourceModule, uID);
	bSuccess = !m_img.IsNull();
	return bSuccess;
}
BOOL CBitmapImage::Load(PVOID pBuf, DWORD dwBufLen)
{
	BOOL bSuccess = FALSE;
	Unload();
	m_hBuffer  = ::GlobalAlloc(GMEM_MOVEABLE, dwBufLen);
	if (m_hBuffer)
	{
		void* pBuffer = ::GlobalLock(m_hBuffer);
		if (pBuffer)
		{
			CopyMemory(pBuffer, pBuf, dwBufLen);

			IStream* pStream = NULL;
			if (::CreateStreamOnHGlobal(m_hBuffer, FALSE, &pStream) == S_OK)
			{
				HRESULT hr = m_img.Load(pStream);
				(hr);
				pStream->Release();

				if (!m_img.IsNull())
				{
					return TRUE;
				}
			}
			::GlobalUnlock(m_hBuffer);
		}
		::GlobalFree(m_hBuffer);
		m_hBuffer = NULL;
	}
	bSuccess = !m_img.IsNull();
	return bSuccess;
}
BOOL CBitmapImage::LoadFromFile(LPCTSTR pszcFile)
{
	BOOL bSuccess = FALSE;
	Unload();
	HRESULT hr = m_img.Load(pszcFile);
	bSuccess = SUCCEEDED(hr);
	return bSuccess;
}
void CBitmapImage::Dispose()
{
	Unload();
}
CBitmapImage::operator HBITMAP()
{
	return (HBITMAP)m_img;
}
void CBitmapImage::Unload()
{
	if (!m_img.IsNull())
	{
		m_img.Destroy();
	}
}
void CBitmapImage::DrawImage(HDC hDC, LPRECT lpSrcRect, LPRECT lpDestRect)
{
	if (!m_img.IsNull())
	{
		CRect rcSrc = lpSrcRect;
		CRect rcDest = lpDestRect;
		m_img.Draw(hDC, rcDest, rcSrc);
	}
}
BOOL CBitmapImage::TransparentBlt(HDC hDestDC, LPRECT lpSrcRect, LPRECT lpDestRect, UINT crTransparent)
{
	if (!m_img.IsNull())
	{
		RECT rcSrc,rcDest;
		rcSrc.left = lpSrcRect->left;
		rcSrc.top = lpSrcRect->top;
		rcSrc.right = lpSrcRect->right;
		rcSrc.bottom = lpSrcRect->bottom;
		rcDest.left = lpDestRect->left;
		rcDest.top = lpDestRect->top;
		rcDest.right = lpDestRect->right;
		rcDest.bottom = lpDestRect->bottom;
		return m_img.TransparentBlt(hDestDC, rcSrc, rcDest,crTransparent);
	}
	else
		return FALSE;
}
int CBitmapImage::GetWidth()
{
	if (!m_img.IsNull())
	{
		return m_img.GetWidth();
	}
	return -1;
}
int CBitmapImage::GetHeight()
{
	if (!m_img.IsNull())
	{
		return m_img.GetHeight();
	}
	return -1;
}